/*
 * elastic.cpp
 *
 *      Author: David Fernandez Lopez
 *      Debris Collector Core
 */

#include <cmath>
#include "elastic.h"

#define MAX_DIST 100.0f
#define PI 3.14159265f

/*******************************************/
/************ General use methods **********/
/*******************************************/

void cross_product(EuclideanVector* v1, EuclideanVector* v2, EuclideanVector* result1, EuclideanVector* result2){
	//v1 x v2
	*result1->x = (*v1->y *(*v2->z)) - (*v1->z *(*v2->y));
	*result1->y = (*v1->z *(*v2->x)) - (*v1->x *(*v2->z));
	*result1->z = (*v1->x *(*v2->y)) - (*v1->y *(*v2->x));
	//v2 x v1
	*result2->x = (*v2->y *(*v1->z)) - (*v2->z *(*v1->y));
	*result2->y = (*v2->z *(*v1->x)) - (*v2->x *(*v1->z));
	*result2->z = (*v2->x *(*v1->y)) - (*v2->y *(*v1->x));
}

void cross_product(EuclideanVector* v1, EuclideanVector* v2, EuclideanVector* result){
	//v1 x v2
	*result->x = (*v1->y *(*v2->z)) - (*v1->z *(*v2->y));
	*result->y = (*v1->z *(*v2->x)) - (*v1->x *(*v2->z));
	*result->z = (*v1->x *(*v2->y)) - (*v1->y *(*v2->x));
}

float scalar_product(EuclideanVector* v1, EuclideanVector* v2){
	return (*v1->x *(*v2->x)) + (*v1->y *(*v2->y)) + (*v1->z *(*v2->z));
}

int choose(EuclideanVector* f0, EuclideanVector* f1, EuclideanVector* v){
	float norm_v = norm(v);
	float cos0_v = scalar_product(f0, v) / (norm(f0)*norm_v);
	float cos1_v = scalar_product(f1, v) / (norm(f1)*norm_v);
	if(cos0_v > cos1_v)
		return 0;
	else
		return 1;
}

void closeness(EuclideanVector* D){
	float factor = MAX_DIST / norm(D);
	*D->x = (factor * (*D->x)) - *D->x;
	*D->y = (factor * (*D->y)) - *D->y;
	*D->z = (factor * (*D->z)) - *D->z;
}

void orbit_speed(ProjectableSpaceObject* so, EuclideanVector* v){
	EuclideanVector* p0 = new EuclideanVector(*so->orbit->starting_point->r * sin(*so->orbit->starting_point->polar*PI/180.0f) * cos(*so->orbit->starting_point->azimuth*PI/180.0f),*so->orbit->starting_point->r * sin(*so->orbit->starting_point->polar*PI/180.0f) * sin(*so->orbit->starting_point->azimuth*PI/180.0f), *so->orbit->starting_point->r * cos(*so->orbit->starting_point->polar*PI/180.0f));
	EuclideanVector* p1 = new EuclideanVector(*so->project(1.0f)->r * sin(*so->project(1.0f)->polar*PI/180.0f) * cos(*so->project(1.0f)->azimuth*PI/180.0f),*so->project(1.0f)->r * sin(*so->project(1.0f)->polar*PI/180.0f) * sin(*so->project(1.0f)->azimuth*PI/180.0f), *so->project(1.0f)->r * cos(*so->project(1.0f)->polar*PI/180.0f));

	*v->x = *p1->x - *p0->x;
	*v->y = *p1->y - *p0->y;
	*v->z = *p1->z - *p0->z;

	delete p0;
	delete p1;
}

void distance_vector(Point* p1, Point* p2, EuclideanVector* v){
	*v->x = (*p2->r * sin(*p2->polar*PI/180.0f) * cos(*p2->azimuth*PI/180.0f)) - (*p1->r * sin(*p1->polar*PI/180.0f) * cos(*p1->azimuth*PI/180.0f));
	*v->y = (*p2->r * sin(*p2->polar*PI/180.0f) * sin(*p2->azimuth*PI/180.0f)) - (*p1->r * sin(*p1->polar*PI/180.0f) * sin(*p1->azimuth*PI/180.0f));
	*v->z = (*p2->r * cos(*p2->polar*PI/180.0f)) - (*p1->r * cos(*p1->polar*PI/180.0f));
}

void quaternion(EuclideanVector* v1, EuclideanVector* v2, float* x, float* y, float* z, float* w){
	EuclideanVector* cross = new EuclideanVector();
	cross_product(v1, v2, cross);

	*x = *cross->x;
	*y = *cross->y;
	*z = *cross->z;
	*w = 1.0f + scalar_product(v1, v2);

	float norm = sqrt(*x*(*x)+*y*(*y)+*z*(*z)+*w*(*w));
	*x = *x/norm;
	*y = *y/norm;
	*z = *z/norm;
	*w = *w/norm;

	delete cross;
}
