/*
 * genetics.h
 *
 *      Author: David Fernandez Lopez
 *      Debris Collector Core
 */

#include <vector>
#include "model.h"

#ifndef GENETICS_H
#define GENETICS_H

using namespace std;

/*
Struct Number
*/
struct Number{
	int number;
	Number* next;
};

/*
Class Roulette
*/
class Roulette {
	public:
		Number* ball;
		Roulette(int max);
		~Roulette();
		int roll(int passes);
};

/*
Class Individual
*/
class Individual{
	public:
		vector<int> order;
		vector<float> time;
		vector<float> cost;
		float* total_cost;
		float* fitness;
		Individual(int size);
		Individual(int size, const vector<ProjectableTargetSO*> &points, Point* robot);
		~Individual();
		float cost_to_next(int current);
		int next(int current);
		float next_time(int current);
		void compute_costs(const vector<ProjectableTargetSO*> &points, Point* robot);
		bool loadValue(int elem, int pos);
		bool loadValue(int elem, float new_time, int pos);
		void loadRandomValue(int pos);
		void mutation(int pos1, int pos2);
		void migration(const vector<ProjectableTargetSO*> &points, Point* robot);
};

/*
Class Population
*/
class Population{
	public:
		float* best_cost;
		vector<Individual*> individuals;
		Population(int size, const vector<ProjectableTargetSO*> &points, Point* robot);
		Population(int size);
		~Population();
		int chooseParent();
		int compute_costs(const vector<ProjectableTargetSO*> &points, Point* robot);
};

float random_float(float a, float b);
float loadTime(Point* a, ProjectableTargetSO* b, float t);

#endif
